require 'csv'

module AdWords

  City2DMA = Struct.new(:city, :criteria_id, :state, :dma_region, :dma_region_code) do

    def ==(other)
      self.criteria_id == other.criteria_id && self.dma_region_code == other.dma_region_code
    end

    def description
      "#{dma_region},#{state}"
    end

    def self.all 
      data.drop(1).map do |row|
        AdWords::City2DMA.new(row[0], row[1].to_i, row[2], row[3], row[4].to_i)
      end
    end

    private 

    def self.data
      adwords_dma_file = 'adwords_api_cities_dma_regions_2014-07-23.csv'
      @data ||= CSV.read(File.join(File.dirname(__FILE__), '..', 'data', adwords_dma_file)) 
    end

  end

  class City2DMADb 

    def initialize
      @city_to_dma_records = {}
      cities = City2DMA.all
      cities.each do |r|
        @city_to_dma_records[r.criteria_id] ||= []
        @city_to_dma_records[r.criteria_id] << r unless @city_to_dma_records[r.criteria_id].include? r
      end
      @city_to_dma_records.freeze
    end

    def size
      @city_to_dma_records.size
    end

    def find_dmas_city_id(criteria_id)
      @city_to_dma_records[criteria_id.to_i] || []
    end

  end

end

