require 'csv'
require_relative 'city_2_dma'

module AdWords

  Location = Struct.new(:criteria_id, :name, :canonical_name, :parent_id, :country_code, :target_type, :status) do 

    POSTAL_CODE = 'Postal Code'
    US = 'US'

    def description
      "#{canonical_name} (#{target_type})"
    end

    def postal_code
      raise "Call postal_code the the record targeting type is: #{target_type}" unless target_type == POSTAL_CODE
      name.rjust(5,'0')
    end

    def us_postal_code?
      target_type == POSTAL_CODE and country_code == US
    end

    def self.all
      data.drop(1).map do |row|
        AdWords::Location.new(row[0].to_i, row[1], row[2], row[3].to_i, row[4], row[5], row[6])
      end
    end

    private 

    def self.data
      adwords_location_file = 'adwords_api_location_criteria_2014-07-23.csv'
      @data ||= CSV.read(File.join(File.dirname(__FILE__), '..', 'data', adwords_location_file)) 
    end
  end

  class LocationDb 

    def initialize
      @location_records = {}
      @location_records_by_name = {}
      @location_records_by_canonical_name = {}

      records = Location.all 
      records.each do |r|  
        @location_records[r.criteria_id] = r
        @location_records_by_name[r.name] = r
        @location_records_by_canonical_name[r.canonical_name] = r
      end
      @location_records.freeze
      @location_records_by_name.freeze
      @location_records_by_canonical_name.freeze

      @city_2_dma = City2DMADb.new
    end

    def criteria_ids
      @location_records.keys
    end

    def find(criteria_id)
      @location_records[criteria_id.to_i]
    end

    def find_by_name(name)
      @location_records_by_name[name]
    end

    def find_by_canonical_name(cname)
      @location_records_by_canonical_name[cname]
    end

    def all_postal_code_records
      @location_records.select do |id, record|
        record.us_postal_code?
      end
    end

    def all_postal_codes
      @postal_code ||= get_call_postal_codes
    end

    def path(criteria_id)
      path = []
      location = find criteria_id
      while location do
        path << location.criteria_id
        location = find location.parent_id
      end 
      path
    end

    def dmas_by_city(criteria_id)
      dmas = @city_2_dma.find_dmas_city_id criteria_id
      dmas.map do |dma|
        find_by_name dma.dma_region
      end
    end

    private 

    def get_call_postal_codes
      codes = []
      @location_records.each do |id, r|
        codes << r.postal_code if r.us_postal_code?
      end
      codes
    end
    
  end

end

