require 'csv'

module AdWords

  PostalCode = Struct.new(:country_code, :postal_code, :city, :state, :state_code) do

    def canonical_name 
      "#{postal_code},#{state_canonical_name}"
    end

    def city_canonical_name
      "#{city},#{state_canonical_name}"
    end

    def state_canonical_name
      "#{state},United States"
    end

    def self.all
      data.drop(1).map do |row|
        AdWords::PostalCode.new(row[0], row[1].rjust(5, '0'), row[2], row[3], row[4])
      end
    end

    def self.data
      postal_code_file = 'postal_code.csv'
      @data ||= CSV.read(File.join(File.dirname(__FILE__), '..', 'data', 'postal_code', 'US', postal_code_file)) 
    end

  end

  class PostalCodeDb 

    def initialize
      @postal_code_records = {}
      PostalCode.all.each do |r|
        @postal_code_records[r.postal_code] ||= []
        @postal_code_records[r.postal_code] << r
      end
      @postal_code_records.freeze
    end

    def size
      @postal_code_records.size
    end

    def find_records_by_postal_code(postal_code)
      @postal_code_records[postal_code]
    end

    def include?(postal_code)
      @postal_code_records.include? postal_code
    end

    def cities_by_postal_code(postal_code)
      if include? postal_code then
        @postal_code_records[postal_code].map do |record|
          record.city_canonical_name
        end
      else 
        []
      end
    end

    def all_postal_codes
      @postal_code_records.keys
    end

  end

end